/*
  @Sudoku solver application
  @Author: Mukul Gandhi
  @Date: 22.03.2006
*/

import java.util.Set;
import java.util.List;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import orbital.util.Setops;

public class SudokuSolver {

    //initialize sudoku board
    static int[][] board = LoadBoard.load();

    /*
      main method of the class
    */
    public static void main(String[] args) {

       //do prelimnary scan of the board
       while(initialScanBoard()) {
	     //dummy loop
	   }

       if (isBoardSolved()) {
	     //print solved puzzle
	     System.out.println("\nsolved puzzle");
	     System.out.println("-----------------");
	     printBoard();
	     System.exit(0);
	   }

       //do deep scan of the board
       Collection options = deepScanBoard();

       //check for the solution
       Iterator it = options.iterator();
       while (it.hasNext()) {
         ArrayList al = (ArrayList)it.next();
         for(int i = 0; i < al.size(); i++) {
           board[((CellInfo)al.get(i)).getI()][((CellInfo)al.get(i)).getJ()] = ((CellInfo)al.get(i)).getNumber();
		 }

		 if (isBoardSolved()) {
		   //print solved puzzle
           System.out.println("\nsolved puzzle");
           System.out.println("--------------");
           printBoard();
           break;
		 }
       }

    }

    //Deep scan the board
    static Collection deepScanBoard() {

      List sets = new ArrayList();

	  Set all = null;

	  for(int i = 0; i < 9; i++) {
		for(int j = 0; j < 9; j++) {
	      if (board[i][j] == 0) {
	        Set sn = findSectionNumbers(i,j);
	        Set rn = findRowNumbers(i);
	        Set cn = findColumnNumbers(j);

	        //set difference
	        all = getAllSet();
	        all.removeAll(sn);
	        Set s1 = all;

	        //set difference
	        all = getAllSet();
	        all.removeAll(rn);
	        Set s2 = all;

	        //set difference
	        all = getAllSet();
	        all.removeAll(cn);
	        Set s3 = all;

	        //set intersection
	        s1.retainAll(s2);
	        s3.retainAll(s1);

  		    Iterator it = s3.iterator();
  		    Collection col = new ArrayList();

  		    while (it.hasNext()) {
			  col.add(new CellInfo(i,j,((Integer)it.next()).intValue()));
		    }

		    sets.add(col);
		  }
		}
	  }

      //cartesian product
      Collection cp = Setops.cross(sets);

	  return cp;

	}


    /*
      Function to check if board is solved
    */
    static boolean isBoardSolved() {

       //check if no cell contains 0
       for(int i = 0; i < 9; i++) {
		 for(int j = 0; j < 9; j++) {
		   if (board[i][j] == 0) {
			 return false;
		   }
		 }
	   }

       Set oneToNine = new HashSet();
       oneToNine.add(new Integer(1));
	   oneToNine.add(new Integer(2));
	   oneToNine.add(new Integer(3));
	   oneToNine.add(new Integer(4));
	   oneToNine.add(new Integer(5));
	   oneToNine.add(new Integer(6));
	   oneToNine.add(new Integer(7));
	   oneToNine.add(new Integer(8));
       oneToNine.add(new Integer(9));

       //check all rows for correctness
	   for(int i = 0; i < 9; i++) {
         Set x = new HashSet();

         x.add(new Integer(board[i][0]));
         x.add(new Integer(board[i][1]));
         x.add(new Integer(board[i][2]));
         x.add(new Integer(board[i][3]));
         x.add(new Integer(board[i][4]));
         x.add(new Integer(board[i][5]));
         x.add(new Integer(board[i][6]));
         x.add(new Integer(board[i][7]));
         x.add(new Integer(board[i][8]));

         if (!x.containsAll(oneToNine)) {
		   return false;
		 }
	   }

       //check all columns for correctness
	   for(int j = 0; j < 9; j++) {
		 Set x = new HashSet();

         x.add(new Integer(board[0][j]));
         x.add(new Integer(board[1][j]));
         x.add(new Integer(board[2][j]));
         x.add(new Integer(board[3][j]));
         x.add(new Integer(board[4][j]));
         x.add(new Integer(board[5][j]));
         x.add(new Integer(board[6][j]));
         x.add(new Integer(board[7][j]));
         x.add(new Integer(board[8][j]));

         if (!x.containsAll(oneToNine)) {
		   return false;
		 }
	   }

       //check section 1
       Set section1 = new HashSet();

       for(int i = 0; i < 3; i++) {
		 for(int j = 0; j < 3; j++) {
           section1.add(new Integer(board[i][j]));
		 }
	   }

	   if (!section1.containsAll(oneToNine)) {
         return false;
	   }

       //check section 2
	   Set section2 = new HashSet();

	   for(int i = 0; i < 3; i++) {
	     for(int j = 3; j < 6; j++) {
	       section2.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section2.containsAll(oneToNine)) {
	     return false;
	   }

       //check section 3
	   Set section3 = new HashSet();

	   for(int i = 0; i < 3; i++) {
	     for(int j = 6; j < 9; j++) {
	       section3.add(new Integer(board[i][j]));
	     }
	   }

	   if (!section3.containsAll(oneToNine)) {
	     return false;
	   }

	   //check section 4
	   Set section4 = new HashSet();

	   for(int i = 3; i < 6; i++) {
	     for(int j = 0; j < 3; j++) {
	       section4.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section4.containsAll(oneToNine)) {
	     return false;
	   }

	   //check section 5
	   Set section5 = new HashSet();

	   for(int i = 3; i < 6; i++) {
	     for(int j = 3; j < 6; j++) {
	       section5.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section5.containsAll(oneToNine)) {
	     return false;
	   }

	   //check section 6
	   Set section6 = new HashSet();

	   for(int i = 3; i < 6; i++) {
	     for(int j = 6; j < 9; j++) {
	       section6.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section6.containsAll(oneToNine)) {
	     return false;
	   }

	   //check section 7
	   Set section7 = new HashSet();

	   for(int i = 6; i < 9; i++) {
	   	 for(int j = 0; j < 3; j++) {
	   	   section7.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section7.containsAll(oneToNine)) {
	     return false;
	   }

	   //check section 8
	   Set section8 = new HashSet();

	   for(int i = 6; i < 9; i++) {
	     for(int j = 3; j < 6; j++) {
	       section8.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section8.containsAll(oneToNine)) {
	     return false;
	   }

       //check section 9
	   Set section9 = new HashSet();

	   for(int i = 6; i < 9; i++) {
	     for(int j = 6; j < 9; j++) {
	       section9.add(new Integer(board[i][j]));
	   	 }
	   }

	   if (!section9.containsAll(oneToNine)) {
	     return false;
	   }

	   //return true status
       return true;
	}

    /*
      Initial scan of the board. Solve as much as possible, without
      backtracking
    */
    static boolean initialScanBoard() {

      boolean scanrequired = false;
      Set all = null;

      for(int i = 0; i < 9; i++) {
		for(int j = 0; j < 9; j++) {
          if (board[i][j] == 0) {
            Set sn = findSectionNumbers(i,j);
            Set rn = findRowNumbers(i);
            Set cn = findColumnNumbers(j);

            //set difference
            all = getAllSet();
            all.removeAll(sn);
            Set s1 = all;

            //set difference
            all = getAllSet();
            all.removeAll(rn);
            Set s2 = all;

            //set difference
            all = getAllSet();
            all.removeAll(cn);
            Set s3 = all;

            //set intersection
            s1.retainAll(s2);
            s3.retainAll(s1);

            if (s3.size() == 1) {
			  Iterator it = s3.iterator();
			  board[i][j] = ((Integer)it.next()).intValue();
			  scanrequired = true;
			}
		  }
		}
	  }

      return scanrequired;
	}

    /*
      Helper function
    */
    static Set getAllSet() {
      Set all = new HashSet();

	  all.add(new Integer(1));
	  all.add(new Integer(2));
	  all.add(new Integer(3));
	  all.add(new Integer(4));
	  all.add(new Integer(5));
	  all.add(new Integer(6));
	  all.add(new Integer(7));
	  all.add(new Integer(8));
      all.add(new Integer(9));

      return all;
	}

    /*
      Helper function. Sudoku board is divided into 9 sections,
      where each section is a 3x3 grid.
    */
    static Set findSectionNumbers(int i, int j) {

      Set section = new HashSet();

      //section 1
      if ((i >=0) && (i <=2) && (j >= 0) && (j <= 2)) {
         section.add(new Integer(board[0][0]));
         section.add(new Integer(board[0][1]));
         section.add(new Integer(board[0][2]));
         section.add(new Integer(board[1][0]));
         section.add(new Integer(board[1][1]));
         section.add(new Integer(board[1][2]));
         section.add(new Integer(board[2][0]));
         section.add(new Integer(board[2][1]));
         section.add(new Integer(board[2][2]));
         section.remove(new Integer(0));
	  }
	  //section 2
      else if ((i >=0) && (i <=2) && (j >= 3) && (j <= 5)) {
         section.add(new Integer(board[0][3]));
		 section.add(new Integer(board[0][4]));
		 section.add(new Integer(board[0][5]));
		 section.add(new Integer(board[1][3]));
		 section.add(new Integer(board[1][4]));
		 section.add(new Integer(board[1][5]));
		 section.add(new Integer(board[2][3]));
		 section.add(new Integer(board[2][4]));
		 section.add(new Integer(board[2][5]));
         section.remove(new Integer(0));
	  }
      //section 3
	  else if ((i >=0) && (i <=2) && (j >= 6) && (j <= 8)) {
	     section.add(new Integer(board[0][6]));
	  	 section.add(new Integer(board[0][7]));
	  	 section.add(new Integer(board[0][8]));
	  	 section.add(new Integer(board[1][6]));
	  	 section.add(new Integer(board[1][7]));
	  	 section.add(new Integer(board[1][8]));
	  	 section.add(new Integer(board[2][6]));
	  	 section.add(new Integer(board[2][7]));
	  	 section.add(new Integer(board[2][8]));
	     section.remove(new Integer(0));
	  }
      //section 4
	  else if ((i >=3) && (i <=5) && (j >= 0) && (j <= 2)) {
	     section.add(new Integer(board[3][0]));
	  	 section.add(new Integer(board[3][1]));
	   	 section.add(new Integer(board[3][2]));
	   	 section.add(new Integer(board[4][0]));
	   	 section.add(new Integer(board[4][1]));
	   	 section.add(new Integer(board[4][2]));
	   	 section.add(new Integer(board[5][0]));
	   	 section.add(new Integer(board[5][1]));
	   	 section.add(new Integer(board[5][2]));
	     section.remove(new Integer(0));
	  }
      //section 5
	  else if ((i >=3) && (i <=5) && (j >= 3) && (j <= 5)) {
	     section.add(new Integer(board[3][3]));
	  	 section.add(new Integer(board[3][4]));
	   	 section.add(new Integer(board[3][5]));
	   	 section.add(new Integer(board[4][3]));
	   	 section.add(new Integer(board[4][4]));
	   	 section.add(new Integer(board[4][5]));
	   	 section.add(new Integer(board[5][3]));
	   	 section.add(new Integer(board[5][4]));
	   	 section.add(new Integer(board[5][5]));
	     section.remove(new Integer(0));
	  }
      //section 6
	  else if ((i >=3) && (i <=5) && (j >= 6) && (j <= 8)) {
	     section.add(new Integer(board[3][6]));
	  	 section.add(new Integer(board[3][7]));
	   	 section.add(new Integer(board[3][8]));
	   	 section.add(new Integer(board[4][6]));
	   	 section.add(new Integer(board[4][7]));
	   	 section.add(new Integer(board[4][8]));
	   	 section.add(new Integer(board[5][6]));
	   	 section.add(new Integer(board[5][7]));
	   	 section.add(new Integer(board[5][8]));
	     section.remove(new Integer(0));
	  }
      //section 7
	  else if ((i >=6) && (i <=8) && (j >= 0) && (j <= 2)) {
	     section.add(new Integer(board[6][0]));
	  	 section.add(new Integer(board[7][1]));
	   	 section.add(new Integer(board[8][2]));
	   	 section.add(new Integer(board[6][0]));
	   	 section.add(new Integer(board[7][1]));
	   	 section.add(new Integer(board[8][2]));
	   	 section.add(new Integer(board[6][0]));
	   	 section.add(new Integer(board[7][1]));
	   	 section.add(new Integer(board[8][2]));
	     section.remove(new Integer(0));
	  }
      //section 8
	  else if ((i >=6) && (i <=8) && (j >= 3) && (j <= 5)) {
	     section.add(new Integer(board[6][3]));
	  	 section.add(new Integer(board[7][4]));
	   	 section.add(new Integer(board[8][5]));
	   	 section.add(new Integer(board[6][3]));
	   	 section.add(new Integer(board[7][4]));
	   	 section.add(new Integer(board[8][5]));
	   	 section.add(new Integer(board[6][3]));
	   	 section.add(new Integer(board[7][4]));
	   	 section.add(new Integer(board[8][5]));
	     section.remove(new Integer(0));
	  }
      //section 9
	  else if ((i >=6) && (i <=8) && (j >= 6) && (j <= 8)) {
	     section.add(new Integer(board[6][6]));
	  	 section.add(new Integer(board[7][7]));
	   	 section.add(new Integer(board[8][8]));
	   	 section.add(new Integer(board[6][6]));
	   	 section.add(new Integer(board[7][7]));
	   	 section.add(new Integer(board[8][8]));
	   	 section.add(new Integer(board[6][6]));
	   	 section.add(new Integer(board[7][7]));
	   	 section.add(new Integer(board[8][8]));
	     section.remove(new Integer(0));
	  }

	  return section;
	}

    /*
      Helper function
    */
    static Set findRowNumbers(int i) {

      Set row = new HashSet();

      row.add(new Integer(board[i][0]));
	  row.add(new Integer(board[i][1]));
	  row.add(new Integer(board[i][2]));
	  row.add(new Integer(board[i][3]));
	  row.add(new Integer(board[i][4]));
	  row.add(new Integer(board[i][5]));
	  row.add(new Integer(board[i][6]));
	  row.add(new Integer(board[i][7]));
	  row.add(new Integer(board[i][8]));
	  row.remove(new Integer(0));

      return row;
	}

	/*
	 Helper function
	*/
	static Set findColumnNumbers(int j) {

	  Set column = new HashSet();

	  column.add(new Integer(board[0][j]));
	  column.add(new Integer(board[1][j]));
	  column.add(new Integer(board[2][j]));
	  column.add(new Integer(board[3][j]));
	  column.add(new Integer(board[4][j]));
	  column.add(new Integer(board[5][j]));
	  column.add(new Integer(board[6][j]));
	  column.add(new Integer(board[7][j]));
	  column.add(new Integer(board[8][j]));
	  column.remove(new Integer(0));

      return column;
	}

    /*
      Print board
    */
    static void printBoard() {
      System.out.println(board[0][0]+" "+board[0][1]+" "+board[0][2]+" "+board[0][3]+" "+board[0][4]+" "+board[0][5]+" "+board[0][6]+" "+board[0][7]+" "+board[0][8]);
      System.out.println(board[1][0]+" "+board[1][1]+" "+board[1][2]+" "+board[1][3]+" "+board[1][4]+" "+board[1][5]+" "+board[1][6]+" "+board[1][7]+" "+board[1][8]);
      System.out.println(board[2][0]+" "+board[2][1]+" "+board[2][2]+" "+board[2][3]+" "+board[2][4]+" "+board[2][5]+" "+board[2][6]+" "+board[2][7]+" "+board[2][8]);
      System.out.println(board[3][0]+" "+board[3][1]+" "+board[3][2]+" "+board[3][3]+" "+board[3][4]+" "+board[3][5]+" "+board[3][6]+" "+board[3][7]+" "+board[3][8]);
      System.out.println(board[4][0]+" "+board[4][1]+" "+board[4][2]+" "+board[4][3]+" "+board[4][4]+" "+board[4][5]+" "+board[4][6]+" "+board[4][7]+" "+board[4][8]);
      System.out.println(board[5][0]+" "+board[5][1]+" "+board[5][2]+" "+board[5][3]+" "+board[5][4]+" "+board[5][5]+" "+board[5][6]+" "+board[5][7]+" "+board[5][8]);
      System.out.println(board[6][0]+" "+board[6][1]+" "+board[6][2]+" "+board[6][3]+" "+board[6][4]+" "+board[6][5]+" "+board[6][6]+" "+board[6][7]+" "+board[6][8]);
      System.out.println(board[7][0]+" "+board[7][1]+" "+board[7][2]+" "+board[7][3]+" "+board[7][4]+" "+board[7][5]+" "+board[7][6]+" "+board[7][7]+" "+board[7][8]);
      System.out.println(board[8][0]+" "+board[8][1]+" "+board[8][2]+" "+board[8][3]+" "+board[8][4]+" "+board[8][5]+" "+board[8][6]+" "+board[8][7]+" "+board[8][8]);
    }

}